require 'spec_helper'

describe RuboCop::Cop::Mirego::ExplicitReturnValue do
  let(:cop) { RuboCop::Cop::Mirego::ExplicitReturnValue.new }

  before { inspect_source(cop, source) }

  describe 'with a return with a value' do
    let(:source) { 'def test; return nil; end' }

    it { expect(cop.offenses.size).to eql(0) }
  end

  describe 'with a return with a value with a conditional' do
    let(:source) { 'def test; return nil unless bool; end' }

    it { expect(cop.offenses.size).to eql(0) }
  end

  describe 'with a return missing a value' do
    let(:source) { 'def test; return; end' }

    specify do
      expect(cop.offenses.size).to eql(1)
      expect(cop.offenses.first.cop_name).to eql('Mirego/ExplicitReturnValue')
    end
  end

  describe 'with a return missing a value with a `unless` conditional' do
    let(:source) { 'def test; return unless bool; end' }

    specify do
      expect(cop.offenses.size).to eql(1)
      expect(cop.offenses.first.cop_name).to eql('Mirego/ExplicitReturnValue')
    end
  end

  describe 'with a return missing a value with a `if` conditional' do
    let(:source) { 'def test; return unless bool; end' }

    specify do
      expect(cop.offenses.size).to eql(1)
      expect(cop.offenses.first.cop_name).to eql('Mirego/ExplicitReturnValue')
    end
  end

  describe 'with a return missing a value with a expression after the return' do
    let(:source) { 'def test; return unless bool; foobar end' }

    specify do
      expect(cop.offenses.size).to eql(1)
      expect(cop.offenses.first.cop_name).to eql('Mirego/ExplicitReturnValue')
    end
  end

  describe 'with a implicit return' do
    let(:source) { 'def test; test; end' }

    it { expect(cop.offenses.size).to eql(0) }
  end
end
